from unittest.mock import patch

import pytest
from llama_index.llms import LocalAI
from llama_index.llms.base import ChatMessage


def test_interfaces() -> None:
    llm = LocalAI(model="placeholder")
    assert llm.class_name() == type(llm).__name__
    assert llm.model == "placeholder"


def test_completion() -> None:
    llm = LocalAI(model="models/placeholder.gguf")

    text = "...\n\nIt was just another day at the office. The sun had ris"
    with patch(
        "llama_index.llms.openai.completion_with_retry",
        return_value={
            "id": "123",
            "object": "text_completion",
            "created": 1696036786,
            "model": "models/placeholder.gguf",
            "choices": [
                {
                    "text": text,
                    "index": 0,
                    "logprobs": None,
                    "finish_reason": "length",
                }
            ],
            "usage": {"prompt_tokens": 13, "completion_tokens": 16, "total_tokens": 29},
        },
    ) as mock_completion:
        response = llm.complete(
            "A long time ago in a galaxy far, far away", use_chat_completions=False
        )
    assert response.text == text
    mock_completion.assert_called_once()
    # Check we remove the max_tokens if unspecified
    assert "max_tokens" not in mock_completion.call_args.kwargs


def test_chat() -> None:
    llm = LocalAI(model="models/placeholder.gguf", globally_use_chat_completions=True)
    content = "placeholder"
    with patch(
        "llama_index.llms.openai.completion_with_retry",
        return_value={
            "id": "123",
            "object": "chat.completion",
            "created": 1696283017,
            "model": "models/placeholder.gguf",
            "choices": [
                {
                    "index": 0,
                    "message": {
                        "role": "assistant",
                        "content": content,
                    },
                    "finish_reason": "length",
                }
            ],
            "usage": {"prompt_tokens": 5, "completion_tokens": 16, "total_tokens": 21},
        },
    ) as mock_chat:
        response = llm.chat([ChatMessage(role="user", content="test message")])
    assert response.message.content == content
    mock_chat.assert_called_once()
    # Check we remove the max_tokens if unspecified
    assert "max_tokens" not in mock_chat.call_args.kwargs


def test_forgetting_kwarg() -> None:
    llm = LocalAI(model="models/placeholder.gguf")

    with patch(
        "llama_index.llms.openai.completion_with_retry", return_value={}
    ) as mock_completion:
        with pytest.raises(NotImplementedError, match="/chat/completions"):
            llm.complete("A long time ago in a galaxy far, far away")
    mock_completion.assert_not_called()



"""Test document summary retrievers."""
from llama_index.indices.document_summary.base import (
    DocumentSummaryIndex,
    DocumentSummaryRetrieverMode,
)
from llama_index.indices.document_summary.retrievers import (
    DocumentSummaryIndexEmbeddingRetriever,
    DocumentSummaryIndexLLMRetriever,
)


def test_embedding_retriever(
    index: DocumentSummaryIndex,
) -> None:
    retriever = index.as_retriever()
    assert isinstance(retriever, DocumentSummaryIndexEmbeddingRetriever)
    results = retriever.retrieve("Test query")
    assert len(results) == 1
    assert results[0].node.ref_doc_id == "doc_4"

    retriever = index.as_retriever(similarity_top_k=2)
    assert isinstance(retriever, DocumentSummaryIndexEmbeddingRetriever)
    results = retriever.retrieve("Test query")
    assert len(results) == 2
    assert results[0].node.ref_doc_id == "doc_3"
    assert results[1].node.ref_doc_id == "doc_4"


def test_llm_retriever(
    index: DocumentSummaryIndex,
) -> None:
    retriever = index.as_retriever(retriever_mode=DocumentSummaryRetrieverMode.LLM)
    assert isinstance(retriever, DocumentSummaryIndexLLMRetriever)
    results = retriever.retrieve("Test query")
    assert len(results) == 1



from typing import Any, Generator

import pytest
from llama_index.llms.base import ChatMessage
from llama_index.llms.konko import Konko
from pytest import MonkeyPatch

try:
    import konko
except ImportError:
    konko = None  # type: ignore


def setup_module() -> None:
    import os

    os.environ["KONKO_API_KEY"] = "ko-" + "a" * 48


def mock_chat_completion(*args: Any, **kwargs: Any) -> dict:
    return {
        "id": "chatcmpl-abc123",
        "object": "chat.completion",
        "created": 1677858242,
        "model": "meta-llama/Llama-2-13b-chat-hf",
        "usage": {"prompt_tokens": 13, "completion_tokens": 7, "total_tokens": 20},
        "choices": [
            {
                "message": {"role": "assistant", "content": "\n\nThis is a test!"},
                "finish_reason": "stop",
                "index": 0,
            }
        ],
    }


def mock_completion(*args: Any, **kwargs: Any) -> dict:
    return {
        "id": "cmpl-uqkvlQyYK7bGYrRHQ0eXlWi7",
        "object": "text_completion",
        "created": 1589478378,
        "model": "meta-llama/Llama-2-13b-chat-hf",
        "choices": [
            {
                "text": "\n\nThis is indeed a test",
                "index": 0,
                "logprobs": None,
                "finish_reason": "length",
            }
        ],
        "usage": {"prompt_tokens": 5, "completion_tokens": 7, "total_tokens": 12},
    }


def mock_chat_completion_stream(
    *args: Any, **kwargs: Any
) -> Generator[dict, None, None]:
    # Example taken from: https://github.com/openai/openai-cookbook/blob/main/examples/How_to_stream_completions.ipynb
    responses = [
        {
            "choices": [
                {"delta": {"role": "assistant"}, "finish_reason": None, "index": 0}
            ],
            "created": 1677825464,
            "id": "chatcmpl-6ptKyqKOGXZT6iQnqiXAH8adNLUzD",
            "model": "meta-llama/Llama-2-13b-chat-hf",
            "object": "chat.completion.chunk",
        },
        {
            "choices": [
                {"delta": {"content": "\n\n"}, "finish_reason": None, "index": 0}
            ],
            "created": 1677825464,
            "id": "chatcmpl-6ptKyqKOGXZT6iQnqiXAH8adNLUzD",
            "model": "gpt-3.5-turbo-0301",
            "object": "chat.completion.chunk",
        },
        {
            "choices": [{"delta": {"content": "2"}, "finish_reason": None, "index": 0}],
            "created": 1677825464,
            "id": "chatcmpl-6ptKyqKOGXZT6iQnqiXAH8adNLUzD",
            "model": "gpt-3.5-turbo-0301",
            "object": "chat.completion.chunk",
        },
        {
            "choices": [{"delta": {}, "finish_reason": "stop", "index": 0}],
            "created": 1677825464,
            "id": "chatcmpl-6ptKyqKOGXZT6iQnqiXAH8adNLUzD",
            "model": "gpt-3.5-turbo-0301",
            "object": "chat.completion.chunk",
        },
    ]
    yield from responses


@pytest.mark.skipif(konko is None, reason="konko not installed")
def test_chat_model_basic_non_openai_model(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr(
        "llama_index.llms.konko.completion_with_retry", mock_chat_completion
    )
    llm = Konko(model="meta-llama/Llama-2-13b-chat-hf")
    prompt = "test prompt"
    message = ChatMessage(role="user", content="test message")

    response = llm.complete(prompt)
    assert response.text is not None

    chat_response = llm.chat([message])
    assert chat_response.message.content is not None


@pytest.mark.skipif(konko is None, reason="konko not installed")
def test_chat_model_basic_openai_model(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr(
        "llama_index.llms.konko.completion_with_retry", mock_chat_completion
    )
    llm = Konko(model="gpt-3.5-turbo")
    prompt = "test prompt"
    message = ChatMessage(role="user", content="test message")

    response = llm.complete(prompt)
    assert response.text is not None

    chat_response = llm.chat([message])
    assert chat_response.message.content is not None


@pytest.mark.skipif(konko is None, reason="konko not installed")
def test_chat_model_streaming(monkeypatch: MonkeyPatch) -> None:
    monkeypatch.setattr(
        "llama_index.llms.konko.completion_with_retry", mock_chat_completion_stream
    )
    llm = Konko(model="meta-llama/Llama-2-13b-chat-hf")
    message = ChatMessage(role="user", content="test message")
    chat_response_gen = llm.stream_chat([message])
    chat_responses = list(chat_response_gen)
    assert chat_responses[-1].message.content is not None


def teardown_module() -> None:
    import os

    del os.environ["KONKO_API_KEY"]



"""Test document summary index."""

from typing import List

from llama_index.indices.document_summary.base import DocumentSummaryIndex
from llama_index.schema import Document


def test_build_index(
    docs: List[Document],
    index: DocumentSummaryIndex,
) -> None:
    """Test build tree."""
    test = index.get_document_summary("doc_1")
    assert test == "summary_query:This is a test v2."
    test4 = index.get_document_summary("doc_4")
    assert test4 == "summary_query:Hello world."

    all_ref_doc_info = index.ref_doc_info
    for idx, (doc_id, ref_doc_info) in enumerate(all_ref_doc_info.items()):
        assert docs[idx].doc_id == doc_id
        assert len(ref_doc_info.node_ids) == 2



from typing import List

import pytest
from llama_index.indices.document_summary.base import DocumentSummaryIndex
from llama_index.indices.service_context import ServiceContext
from llama_index.response_synthesizers import get_response_synthesizer
from llama_index.schema import Document

from tests.mock_utils.mock_prompts import MOCK_REFINE_PROMPT, MOCK_TEXT_QA_PROMPT


@pytest.fixture()
def docs() -> List[Document]:
    return [
        Document(text="This is a test v2.", id_="doc_1"),
        Document(text="This is another test.", id_="doc_2"),
        Document(text="This is a test.", id_="doc_3"),
        Document(text="Hello world.", id_="doc_4"),
    ]


@pytest.fixture()
def index(
    docs: List[Document], mock_service_context: ServiceContext
) -> DocumentSummaryIndex:
    response_synthesizer = get_response_synthesizer(
        text_qa_template=MOCK_TEXT_QA_PROMPT,
        refine_template=MOCK_REFINE_PROMPT,
        callback_manager=mock_service_context.callback_manager,
    )
    return DocumentSummaryIndex.from_documents(
        docs,
        service_context=mock_service_context,
        response_synthesizer=response_synthesizer,
        summary_query="summary_query",
    )
